import { Command, CommandRunOptions } from '../../../structures/Command';
import type { DenkyClient } from '../../../types/Client';

const EMOJIS: Record<string, string> = {
  A: '🇦',
  B: '🇧',
  C: '🇨',
  D: '🇩',
  E: '🇪',
  F: '🇫',
  G: '🇬',
  H: '🇭',
  I: '🇮',
  J: '🇯',
  K: '🇰',
  L: '🇱',
  M: '🇲',
  N: '🇳',
  O: '🇴',
  P: '🇵',
  Q: '🇶',
  R: '🇷',
  S: '🇸',
  T: '🇹',
  U: '🇺',
  V: '🇻',
  W: '🇼',
  X: '🇽',
  Y: '🇾',
  Z: '🇿',
  '!': '❗',
  ' ': '  ',
  '?': '❓',
  $: '💲',
  '-': '➖',
  '+': '➕',
  '#': '#️⃣',
  '*': '*️⃣',
  '0': '0️⃣',
  '1': '1️⃣',
  '2': '2️⃣',
  '3': '3️⃣',
  '4': '4️⃣',
  '5': '5️⃣',
  '6': '6️⃣',
  '7': '7️⃣',
  '8': '8️⃣',
  '9': '9️⃣'
};

export default class TextEmojifySubCommand extends Command {
  constructor(client: DenkyClient) {
    super(client);
    this.rawName = '';
    this.config = {
      autoDefer: true,
      ephemeral: false,
      showInHelp: false,
      guildOnly: true
    };
    this.permissions = { bot: [] };
  }

  override async run({ interaction }: CommandRunOptions) {
    await interaction.deferReply({ ephemeral: true });

    const text = interaction.options.getString('text', true);
    const emojiText = text
      .split('')
      .map(str => {
        const emoji = EMOJIS[str.toUpperCase()];
        if (emoji) return `${emoji} `;
        return undefined;
      })
      .filter(t => t !== undefined)
      .join('')
      .slice(0, 1500);
    interaction.editReply(emojiText);
  }
}
